@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2016
+  Copyright (c) 2006 - 2017
   CLST - Radboud University
   ILK  - Tilburg University
 
@@ -30,10 +30,12 @@
 #include <string>
 #include <map>
 #include <iosfwd>
+#include "unicode/ucnv.h"
 #include "unicode/ustream.h"
 #include "unicode/uchar.h"
 #include "unicode/unistr.h"
 #include "unicode/normlzr.h"
+#include "unicode/regex.h"
 
 namespace Tokenizer {
 
@@ -60,6 +62,26 @@ namespace Tokenizer {
     std::map<UChar, UnicodeString> the_map;
   };
 
+  class UnicodeRegexMatcher {
+  public:
+    UnicodeRegexMatcher( const UnicodeString&, const UnicodeString& name="" );
+    ~UnicodeRegexMatcher();
+    bool match_all( const UnicodeString&, UnicodeString&, UnicodeString&  );
+    const UnicodeString get_match( unsigned int ) const;
+    int NumOfMatches() const;
+    int split( const UnicodeString&, std::vector<UnicodeString>& );
+    UnicodeString Pattern() const;
+  private:
+    UnicodeRegexMatcher( const UnicodeRegexMatcher& );  // inhibit copies
+    UnicodeRegexMatcher& operator=( const UnicodeRegexMatcher& ); // inhibit copies
+    std::string failString;
+    RegexPattern *pattern;
+    RegexMatcher *matcher;
+    UnicodeRegexMatcher();
+    std::vector<UnicodeString> results;
+    const UnicodeString _name;
+  };
+
 } // namespace
 
 #endif // UCTO_UNICODE_H
