@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2016
+  Copyright (c) 2006 - 2017
   CLST - Radboud University
   ILK  - Tilburg University
 
@@ -34,6 +34,10 @@
 #include "ticcutils/StringOps.h"
 #include "libfolia/folia.h"
 #include "ticcutils/CommandLine.h"
+#include "ticcutils/PrettyPrint.h"
+#include "ucto/unicode.h"
+#include "ucto/textcat.h"
+#include "ucto/setting.h"
 #include "ucto/tokenize.h"
 #include <unistd.h>
 
@@ -44,34 +48,35 @@ void usage(){
   cerr << "Usage: " << endl;
   cerr << "\tucto [[options]] [input-file] [[output-file]]"  << endl
        << "Options:" << endl
-       << "\t-c <configfile> - Explicitly specify a configuration file" << endl
-       << "\t-d <value>      - set debug level" << endl
-       << "\t-e <string>     - set input encoding (default UTF8)" << endl
-       << "\t-N <string>     - set output normalization (default NFC)" << endl
-       << "\t-f              - Disable filtering of special characters" << endl
-       << "\t-h or --help    - this message" << endl
-       << "\t-L <language>   - Automatically selects a configuration file by language code" << endl
-       << "\t-l              - Convert to all lowercase" << endl
-       << "\t-u              - Convert to all uppercase" << endl
-       << "\t-n              - One sentence per line (output)" << endl
-       << "\t-m              - One sentence per line (input)" << endl
-       << "\t-v              - Verbose mode" << endl
-       << "\t-s <string>     - End-of-Sentence marker (default: <utt>)" << endl
-       << "\t--passthru      - Don't tokenize, but perform input decoding and simple token role detection" << endl
+       << "\t-c <configfile>  - Explicitly specify a configuration file" << endl
+       << "\t-d <value>       - set debug level" << endl
+       << "\t-e <string>      - set input encoding (default UTF8)" << endl
+       << "\t-N <string>      - set output normalization (default NFC)" << endl
+       << "\t-f               - Disable filtering of special characters" << endl
+       << "\t-h or --help     - this message" << endl
+       << "\t-L <language>    - Automatically selects a configuration file by language code. (default 'generic')" << endl
+       << "\t-l               - Convert to all lowercase" << endl
+       << "\t-u               - Convert to all uppercase" << endl
+       << "\t-n               - One sentence per line (output)" << endl
+       << "\t-m               - One sentence per line (input)" << endl
+       << "\t-v               - Verbose mode" << endl
+       << "\t-s <string>      - End-of-Sentence marker (default: <utt>)" << endl
+       << "\t--passthru       - Don't tokenize, but perform input decoding and simple token role detection" << endl
        << "\t--normalize=<class1>,class2>,... " << endl
-       << "\t                - For class1, class2, etc. output the class tokens instead of the tokens itself." << endl
-       << "\t--filterpunct   - remove all punctuation from the output" << endl
-       << "\t-P              - Disable paragraph detection" << endl
-       << "\t-S              - Disable sentence detection!" << endl
-       << "\t-Q              - Enable quote detection (experimental)" << endl
-       << "\t-V or --version - Show version information" << endl
-       << "\t-x <DocID>      - Output FoLiA XML, use the specified Document ID (obsolete)" << endl
-       << "\t-F              - Input file is in FoLiA XML. All untokenised sentences will be tokenised." << endl
-       << "\t-X              - Output FoLiA XML, use the Document ID specified with --id=" << endl
-       << "\t--id <DocID>    - use the specified Document ID to label the FoLia doc." << endl
-       << "\t--textclass <class> - use the specified class to search text in the the FoLia doc. (deprecated. use --inputclass)" << endl
-       << "\t--inputclass <class> - use the specified class to search text in the the FoLia doc." << endl
-       << "\t--outputclass <class> - use the specified class to output text in the the FoLia doc. (default is 'current'. changing this is dangerous!)" << endl
+       << "\t                 - For class1, class2, etc. output the class tokens instead of the tokens itself." << endl
+       << "\t--filterpunct    - remove all punctuation from the output" << endl
+       << "\t--detectlanguages=<lang1,lang2,..langn> - try to detect languages. Default = 'lang1'" << endl
+       << "\t-P               - Disable paragraph detection" << endl
+       << "\t-S               - Disable sentence detection!" << endl
+       << "\t-Q               - Enable quote detection (experimental)" << endl
+       << "\t-V or --version  - Show version information" << endl
+       << "\t-x <DocID>       - Output FoLiA XML, use the specified Document ID (obsolete)" << endl
+       << "\t-F               - Input file is in FoLiA XML. All untokenised sentences will be tokenised." << endl
+       << "\t-X               - Output FoLiA XML, use the Document ID specified with --id=" << endl
+       << "\t--id <DocID>     - use the specified Document ID to label the FoLia doc." << endl
+       << "\t--textclass <class> - use the specified class to search text in the FoLia doc. (deprecated. use --inputclass)" << endl
+       << "\t--inputclass <class> - use the specified class to search text in the FoLia doc." << endl
+       << "\t--outputclass <class> - use the specified class to output text in the FoLia doc. (default is 'current'. changing this is dangerous!)" << endl
        << "\t                  (-x and -F disable usage of most other options: -nPQVsS)" << endl;
 }
 
@@ -95,17 +100,17 @@ int main( int argc, char *argv[] ){
   string outputclass = "current";
   string normalization = "NFC";
   string inputEncoding = "UTF-8";
-  string cfile = "tokconfig-en";
+  vector<string> language_list;
+  string cfile;
   string ifile;
   string ofile;
   string c_file;
-  string L_file;
   bool passThru = false;
   string norm_set_string;
 
   try {
     TiCC::CL_Options Opts( "d:e:fhlPQunmN:vVSL:c:s:x:FX",
-			   "filterpunct,passthru,textclass:,inputclass:,outputclass:,normalize:,id:,version,help");
+			   "filterpunct,passthru,textclass:,inputclass:,outputclass:,normalize:,id:,version,help,detectlanguages:");
     Opts.init(argc, argv );
     if ( Opts.extract( 'h' )
 	 || Opts.extract( "help" ) ){
@@ -127,7 +132,6 @@ int main( int argc, char *argv[] ){
     splitsentences = !Opts.extract( 'S' );
     xmlin = Opts.extract( 'F' );
     quotedetection = Opts.extract( 'Q' );
-    Opts.extract( 'c', c_file );
     Opts.extract( 's', eosmarker );
     touppercase = Opts.extract( 'u' );
     tolowercase = Opts.extract( 'l' );
@@ -169,8 +173,72 @@ int main( int argc, char *argv[] ){
 	throw TiCC::OptionError( "invalid value for -d: " + value );
       }
     }
-    if ( Opts.extract('L', value ) ){
-      L_file = "tokconfig-" + value;
+    if ( Opts.is_present('L') ) {
+      if ( Opts.is_present('c') ){
+	cerr << "Error: -L and -c options conflict. Use only one of them." << endl;
+	return EXIT_FAILURE;
+      }
+      else if ( Opts.is_present( "detectlanguages" ) ){
+	cerr << "Error: -L and --detectlanguages options conflict. Use only one of them." << endl;
+	return EXIT_FAILURE;
+      }
+    }
+    else if ( Opts.is_present( 'c' )
+	      && Opts.is_present( "detectlanguages" ) ){
+      cerr << "Error: -c and --detectlanguages options conflict. Use only one of them." << endl;
+      return EXIT_FAILURE;
+    }
+
+    Opts.extract( 'c', c_file );
+    string languages;
+    Opts.extract( "detectlanguages", languages );
+    bool do_language_detect = !languages.empty();
+    if ( do_language_detect ){
+      if ( TiCC::split_at( languages, language_list, "," ) < 1 ){
+	throw TiCC::OptionError( "invalid language list: " + languages );
+      }
+    }
+    else {
+      string language;
+      if ( Opts.extract('L', language ) ){
+	// support some backward compatability to old ISO 639-1 codes
+	if ( language == "nl" ){
+	  language = "nld";
+	}
+	else if ( language == "de" ){
+	  language = "deu";
+	}
+	else if ( language == "fr" ){
+	  language = "fra";
+	}
+	else if ( language == "pt" ){
+	  language = "por";
+	}
+	else if ( language == "es" ){
+	  language = "spa";
+	}
+	else if ( language == "fy" ){
+	  language = "fry";
+	}
+	else if ( language == "se" ){
+	  language = "swe";
+	}
+	else if ( language == "en" ){
+	  language = "eng";
+	}
+	else if ( language == "it" ){
+	  language = "ita";
+	}
+	else if ( language == "ru" ){
+	  language = "rus";
+	}
+	else if ( language == "tr" ){
+	  language = "tur";
+	}
+      }
+      if ( !language.empty() ){
+	language_list.push_back( language );
+      }
     }
     Opts.extract("normalize", norm_set_string );
     if ( !Opts.empty() ){
@@ -188,20 +256,14 @@ int main( int argc, char *argv[] ){
   catch( const TiCC::OptionError& e ){
     cerr << "ucto: " << e.what() << endl;
     usage();
-    exit(EXIT_FAILURE);
+    return EXIT_FAILURE;
   }
 
   if ( !passThru ){
-    if ( !c_file.empty() && !L_file.empty()) {
-      cerr << "Error: -L and -c options conflict. Use only one of them." << endl;
-      return EXIT_FAILURE;
-    }
     if ( !c_file.empty() ){
       cfile = c_file;
     }
-    else if ( !L_file.empty() )
-      cfile = L_file;
-    else {
+    else if ( language_list.empty() ){
       cfile = "tokconfig-generic";
     }
   }
@@ -247,7 +309,16 @@ int main( int argc, char *argv[] ){
     }
     else {
       // init exept for passthru mode
-      tokenizer.init( cfile );
+      if ( !cfile.empty() ){
+	if ( !tokenizer.init( cfile ) ){
+	  return EXIT_FAILURE;
+	}
+      }
+      else {
+	if ( !tokenizer.init( language_list ) ){
+	  return EXIT_FAILURE;
+	}
+      }
     }
 
     tokenizer.setEosMarker( eosmarker );
